import React from 'react'
import ConsentManager, { open } from '@hashicorp/react-consent-manager'
import defaultCategories from './categories'
import ossPreset from './presets/oss'
import enterprisePreset from './presets/enterprise'

export default function createConsentManager({
  segmentWriteKey = process.env.SEGMENT_WRITE_KEY,
  preset,
  segmentServices,
  otherServices,
  categories,
  forceShow,
}) {
  // if hashi env is present, check against it. if not, fall back to checking node env
  const isProd = process.env.HASHI_ENV
    ? process.env.HASHI_ENV === 'production'
    : process.env.NODE_ENV === 'production'

  if (!process.env.HASHI_ENV)
    console.warn(
      'Consent manager expects the "HASHI_ENV" environment variable to be set in order to ensure proper analytics tracking. Please make sure "HASHI_ENV" is set appropriately within your app.'
    )

  // set the correct segment key based on environment
  const segmentKey = isProd
    ? segmentWriteKey
    : '0EXTgkNx0Ydje2PGXVbRhpKKoe5wtzcE'

  // same for the utility server
  let utilityServerRoot = isProd
    ? 'https://util.hashicorp.com'
    : 'https://hashicorp-web-util-staging.herokuapp.com'

  // allow per-project utility server override. useful for local development
  if (process.env.UTIL_SERVER) {
    utilityServerRoot = process.env.UTIL_SERVER.replace(/\/$/, '')
  }

  // next we build the config objct, kicking it off with the default values
  let config = {
    version: 4,
    companyName: 'HashiCorp',
    privacyPolicyLink: 'https://www.hashicorp.com/privacy',
    segmentWriteKey: segmentKey,
    utilServerRoot: utilityServerRoot,
    categories: defaultCategories,
    forceShow,
  }

  // add preset values if present
  if (preset === 'oss') config = { ...config, ...ossPreset }
  if (preset === 'enterprise') config = { ...config, ...enterprisePreset }

  // add any other custom values nondestructively if present
  if (segmentServices) {
    if (!config.segmentServices) config.segmentServices = []
    config.segmentServices.push(...segmentServices)
  }
  if (otherServices) {
    if (!config.additionalServices) config.additionalServices = []
    config.additionalServices.push(...otherServices)
  }
  if (categories) config.categories.push(...categories)

  // finally, we return a HOC that will render the fully configured consent manager
  return {
    ConsentManager: function ConsentManagerWrapper() {
      return <ConsentManager {...config} />
    },
    openConsentManager: open,
  }
}
